@@ -20,17 +20,18 @@
 
 """
 
+from collections import namedtuple
 from collections import OrderedDict
 import copy
 import datetime
 import hashlib
 import inspect
 import logging
-import mock
-from oslo_utils import versionutils as vutils
-import six
+from unittest import mock
 
 import fixtures
+from oslo_utils import versionutils as vutils
+
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
@@ -124,14 +125,14 @@ class FakeIndirectionAPI(base.VersionedObjectIndirectionAPI):
         kwargs = dict(
             [(argname, self._ser.deserialize_entity(
                 context, self._ser.serialize_entity(context, arg)))
-             for argname, arg in six.iteritems(kwargs)])
+             for argname, arg in kwargs.items()])
         return args, kwargs
 
     def object_action(self, context, objinst, objmethod, args, kwargs):
         objinst = self._ser.deserialize_entity(
             context, self._ser.serialize_entity(
                 context, objinst))
-        objmethod = six.text_type(objmethod)
+        objmethod = str(objmethod)
         args, kwargs = self._canonicalize_args(context, args, kwargs)
         original = objinst.obj_clone()
         with mock.patch('oslo_versionedobjects.base.VersionedObject.'
@@ -143,9 +144,9 @@ class FakeIndirectionAPI(base.VersionedObjectIndirectionAPI):
 
     def object_class_action(self, context, objname, objmethod, objver,
                             args, kwargs):
-        objname = six.text_type(objname)
-        objmethod = six.text_type(objmethod)
-        objver = six.text_type(objver)
+        objname = str(objname)
+        objmethod = str(objmethod)
+        objver = str(objver)
         args, kwargs = self._canonicalize_args(context, args, kwargs)
         cls = base.VersionedObject.obj_class_from_name(objname, objver)
         with mock.patch('oslo_versionedobjects.base.VersionedObject.'
@@ -158,10 +159,9 @@ class FakeIndirectionAPI(base.VersionedObjectIndirectionAPI):
 
     def object_class_action_versions(self, context, objname, objmethod,
                                      object_versions, args, kwargs):
-        objname = six.text_type(objname)
-        objmethod = six.text_type(objmethod)
-        object_versions = {six.text_type(o): six.text_type(v)
-                           for o, v in object_versions.items()}
+        objname = str(objname)
+        objmethod = str(objmethod)
+        object_versions = {str(o): str(v) for o, v in object_versions.items()}
         args, kwargs = self._canonicalize_args(context, args, kwargs)
         objver = object_versions[objname]
         cls = base.VersionedObject.obj_class_from_name(objname, objver)
@@ -198,6 +198,31 @@ class ObjectHashMismatch(Exception):
             ','.join(set(self.expected.keys() + self.actual.keys())))
 
 
+CompatArgSpec = namedtuple(
+    'ArgSpec', ('args', 'varargs', 'keywords', 'defaults'))
+
+
+def get_method_spec(method):
+    """Get a stable and compatible method spec.
+
+    Newer features in Python3 (kw-only arguments and annotations) are
+    not supported or representable with inspect.getargspec() but many
+    object hashes are already recorded using that method. This attempts
+    to return something compatible with getargspec() when possible (i.e.
+    when those features are not used), and otherwise just returns the
+    newer getfullargspec() representation.
+    """
+    fullspec = inspect.getfullargspec(method)
+    if any([fullspec.kwonlyargs, fullspec.kwonlydefaults,
+            fullspec.annotations]):
+        # Method uses newer-than-getargspec() features, so return the
+        # newer full spec
+        return fullspec
+    else:
+        return CompatArgSpec(fullspec.args, fullspec.varargs,
+                             fullspec.varkw, fullspec.defaults)
+
+
 class ObjectVersionChecker(object):
     def __init__(self, obj_classes=base.VersionedObjectRegistry.obj_classes()):
         self.obj_classes = obj_classes
@@ -206,8 +231,8 @@ class ObjectVersionChecker(object):
         """Follow a chain of remotable things down to the original function."""
         if isinstance(thing, classmethod):
             return self._find_remotable_method(cls, thing.__get__(None, cls))
-        elif (inspect.ismethod(thing)
-              or inspect.isfunction(thing)) and hasattr(thing, 'remotable'):
+        elif (inspect.ismethod(thing) or
+              inspect.isfunction(thing)) and hasattr(thing, 'remotable'):
             return self._find_remotable_method(cls, thing.original_fn,
                                                parent_was_remotable=True)
         elif parent_was_remotable:
@@ -229,7 +254,7 @@ class ObjectVersionChecker(object):
                or isinstance(thing, classmethod):
                 method = self._find_remotable_method(obj_class, thing)
                 if method:
-                    methods.append((name, inspect.getargspec(method)))
+                    methods.append((name, get_method_spec(method)))
         methods.sort()
         # NOTE(danms): Things that need a version bump are any fields
         # and their types, or the signatures of any remotable methods.
@@ -247,7 +272,7 @@ class ObjectVersionChecker(object):
             relevant_data += extra_data_func(obj_class)
 
         fingerprint = '%s-%s' % (obj_class.VERSION, hashlib.md5(
-            six.binary_type(repr(relevant_data).encode())).hexdigest())
+            bytes(repr(relevant_data).encode())).hexdigest())
         return fingerprint
 
     def get_hashes(self, extra_data_func=None):
@@ -320,8 +345,9 @@ class ObjectVersionChecker(object):
         kwargs = {'version_manifest': manifest} if manifest else {}
         for n in range(version[1] + 1):
             test_version = '%d.%d' % (version[0], n)
-            LOG.info('testing obj: %s version: %s' %
-                     (obj_class.obj_name(), test_version))
+            # Run the test with OS_DEBUG=True to see this.
+            LOG.debug('testing obj: %s version: %s' %
+                      (obj_class.obj_name(), test_version))
             kwargs['target_version'] = test_version
             obj_class(*init_args, **init_kwargs).obj_to_primitive(**kwargs)
 
@@ -369,12 +395,13 @@ class ObjectVersionChecker(object):
             for my_version, child_version in versions:
                 _my_version = vutils.convert_version_to_tuple(my_version)
                 _ch_version = vutils.convert_version_to_tuple(child_version)
-                assert (last_my_version < _my_version
-                        and last_child_version <= _ch_version), \
-                    ('Object %s relationship '
-                     '%s->%s for field %s is out of order') % (
-                         obj_class.obj_name(), my_version,
-                         child_version, field)
+                if not (last_my_version < _my_version and
+                        last_child_version <= _ch_version):
+                    raise AssertionError(('Object %s relationship %s->%s for '
+                                          'field %s is out of order') % (
+                                              obj_class.obj_name(),
+                                              my_version, child_version,
+                                              field))
                 last_my_version = _my_version
                 last_child_version = _ch_version
 
