@@ -20,22 +20,22 @@ Some black magic for inline callbacks.
 
 """
 
-import eventlet
-eventlet.monkey_patch(os=False)
+import eventlet  # noqa
+eventlet.monkey_patch(os=False)  # noqa
 
-import inspect
-import mock
-import os
+import functools   # noqa: E402
+import inspect   # noqa: E402
+import os   # noqa: E402
+from unittest import mock   # noqa: E402
 
-import fixtures
-from oslo_concurrency import lockutils
-from oslo_config import cfg
-from oslo_config import fixture as config_fixture
-from oslo_log.fixture import logging_error
-import six
-import testtools
+import fixtures   # noqa: E402
+from oslo_concurrency import lockutils   # noqa: E402
+from oslo_config import cfg   # noqa: E402
+from oslo_config import fixture as config_fixture   # noqa: E402
+from oslo_log.fixture import logging_error   # noqa: E402
+import testtools   # noqa: E402
 
-from oslo_versionedobjects.tests import obj_fixtures
+from oslo_versionedobjects.tests import obj_fixtures   # noqa: E402
 
 
 CONF = cfg.CONF
@@ -54,7 +54,7 @@ class skipIf(object):
         condition = self.condition
         reason = self.reason
         if inspect.isfunction(func_or_cls):
-            @six.wraps(func_or_cls)
+            @functools.wraps(func_or_cls)
             def wrapped(*args, **kwargs):
                 if condition:
                     raise testtools.TestCase.skipException(reason)
@@ -64,7 +64,7 @@ class skipIf(object):
         elif inspect.isclass(func_or_cls):
             orig_func = getattr(func_or_cls, 'setUp')
 
-            @six.wraps(orig_func)
+            @functools.wraps(orig_func)
             def new_func(self, *args, **kwargs):
                 if condition:
                     raise testtools.TestCase.skipException(reason)
@@ -95,6 +95,7 @@ def _patch_mock_to_raise_for_invalid_assert_calls():
     mock.Mock.__getattr__ = raise_for_invalid_assert_calls(
         mock.Mock.__getattr__)
 
+
 # NOTE(gibi): needs to be called only once at import time
 # to patch the mock lib
 _patch_mock_to_raise_for_invalid_assert_calls()
