@@ -14,12 +14,11 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 import warnings
 
 import iso8601
-import mock
 import netaddr
-import six
 import testtools
 
 from oslo_versionedobjects import _utils
@@ -159,8 +158,6 @@ class TestString(TestField):
         self.field = fields.StringField()
         self.coerce_good_values = [
             ('foo', 'foo'), (1, '1'), (1.0, '1.0'), (True, 'True')]
-        if six.PY2:
-            self.coerce_good_values += [(long(1), '1')]  # noqa
         self.coerce_bad_values = [None]
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
@@ -295,11 +292,12 @@ class TestUUID(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
-    def test_validation_enabled(self):
-
-        self.test_coerce_good_values()
-        self.test_from_primitive()
-        self.test_to_primitive()
+    @mock.patch('warnings.warn')
+    def test_coerce_good_values(self, mock_warn):
+        super().test_coerce_good_values()
+        mock_warn.assert_has_calls(
+            [mock.call(mock.ANY, FutureWarning)] * 8,
+        )
 
     def test_validation_warning_can_be_escalated_to_exception(self):
         warnings.filterwarnings(action='error')
@@ -851,6 +849,24 @@ class TestListOfIntegers(TestField):
         self.assertEqual('[[1, 2]]', self.field.stringify([[1, 2]]))
 
 
+class TestListOfUUIDField(TestField):
+    def setUp(self):
+        super(TestListOfUUIDField, self).setUp()
+        self.field = fields.ListOfUUIDField()
+        self.uuid1 = '6b2097ea-d0e3-44dd-b131-95472b3ea8fd'
+        self.uuid2 = '478c193d-2533-4e71-ab2b-c7683f67d7f9'
+        self.coerce_good_values = [([self.uuid1, self.uuid2],
+                                    [self.uuid1, self.uuid2])]
+        # coerce_bad_values is intentionally ignored since the UUID field
+        # allows non-UUID values for now. See TestUUIDField for examples.
+        self.to_primitive_values = [([self.uuid1], [self.uuid1])]
+        self.from_primitive_values = [([self.uuid1], [self.uuid1])]
+
+    def test_stringify(self):
+        self.assertEqual('[%s,%s]' % (self.uuid1, self.uuid2),
+                         self.field.stringify([self.uuid1, self.uuid2]))
+
+
 class TestLocalMethods(test.TestCase):
     @mock.patch.object(obj_base.LOG, 'exception')
     def test__make_class_properties_setter_value_error(self, mock_log):
@@ -1043,7 +1059,7 @@ class TestObject(TestField):
         ex = self.assertRaises(ValueError, self.field.coerce,
                                'obj', 'attr', [{}])
         self.assertEqual('An object of type TestableObject is required '
-                         'in field attr, not a list', six.text_type(ex))
+                         'in field attr, not a list', str(ex))
 
     def test_get_schema(self):
         self.assertEqual(
@@ -1211,3 +1227,41 @@ class TestIPV6Network(TestField):
         invalid_vals = [x for x in self.coerce_bad_values]
         for invalid_val in invalid_vals:
             self.assertNotRegex(str(invalid_val), pattern)
+
+
+class FakeCounter(object):
+    def __init__(self):
+        self.n = 0
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        if self.n <= 4:
+            self.n += 1
+            return self.n
+        else:
+            raise StopIteration
+
+
+class TestListTypes(test.TestCase):
+
+    def test_regular_list(self):
+        fields.List(fields.Integer).coerce(None, None, [1, 2])
+
+    def test_non_iterable(self):
+        self.assertRaises(ValueError,
+                          fields.List(fields.Integer).coerce, None, None, 2)
+
+    def test_string_iterable(self):
+        self.assertRaises(ValueError,
+                          fields.List(fields.Integer).coerce, None, None,
+                          'hello')
+
+    def test_mapping_iterable(self):
+        self.assertRaises(ValueError,
+                          fields.List(fields.Integer).coerce, None, None,
+                          {'a': 1, 'b': 2})
+
+    def test_iter_class(self):
+        fields.List(fields.Integer).coerce(None, None, FakeCounter())
