@@ -16,10 +16,10 @@ import collections
 import copy
 import datetime
 import hashlib
+import inspect
+from unittest import mock
 
 import iso8601
-import mock
-import six
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import exception
@@ -190,7 +190,7 @@ class TestObjectComparators(test.TestCase):
     def test_compare_obj_with_dt(self):
         mock_test = mock.Mock()
         mock_test.assertEqual = mock.Mock()
-        dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.Utc())
+        dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.UTC)
         replaced_dt = dt.replace(tzinfo=None)
         my_obj = self.MyComparedObjectWithTZ(tzfield=dt)
         my_db_obj = {'tzfield': replaced_dt}
@@ -530,7 +530,7 @@ class TestObjectVersionChecker(test.TestCase):
         MyObject.VERSION = '1.1'
         argspec = 'vulpix'
 
-        with mock.patch('inspect.getargspec') as mock_gas:
+        with mock.patch.object(fixture, 'get_method_spec') as mock_gas:
             mock_gas.return_value = argspec
             fp = self.ovc._get_fingerprint(MyObject.__name__)
 
@@ -538,7 +538,7 @@ class TestObjectVersionChecker(test.TestCase):
         exp_methods = sorted([('remotable_method', argspec),
                               ('remotable_classmethod', argspec)])
         expected_relevant_data = (exp_fields, exp_methods)
-        expected_hash = hashlib.md5(six.binary_type(repr(
+        expected_hash = hashlib.md5(bytes(repr(
             expected_relevant_data).encode())).hexdigest()
         expected_fp = '%s-%s' % (MyObject.VERSION, expected_hash)
 
@@ -553,7 +553,7 @@ class TestObjectVersionChecker(test.TestCase):
         MyObject.child_versions = child_versions
         argspec = 'onix'
 
-        with mock.patch('inspect.getargspec') as mock_gas:
+        with mock.patch.object(fixture, 'get_method_spec') as mock_gas:
             mock_gas.return_value = argspec
             fp = self.ovc._get_fingerprint(MyObject.__name__)
 
@@ -564,7 +564,7 @@ class TestObjectVersionChecker(test.TestCase):
             child_versions.items()))
         exp_relevant_data = (exp_fields, exp_methods, exp_child_versions)
 
-        expected_hash = hashlib.md5(six.binary_type(repr(
+        expected_hash = hashlib.md5(bytes(repr(
             exp_relevant_data).encode())).hexdigest()
         expected_fp = '%s-%s' % (MyObject.VERSION, expected_hash)
 
@@ -594,13 +594,27 @@ class TestObjectVersionChecker(test.TestCase):
         exp_extra_data = ExtraDataObj
         exp_relevant_data = (exp_fields, exp_methods, exp_extra_data)
 
-        expected_hash = hashlib.md5(six.binary_type(repr(
+        expected_hash = hashlib.md5(bytes(repr(
             exp_relevant_data).encode())).hexdigest()
         expected_fp = '%s-%s' % (ExtraDataObj.VERSION, expected_hash)
 
         self.assertEqual(expected_fp, fp, "_get_fingerprint() did not "
                                           "generate a correct fingerprint.")
 
+    def test_get_fingerprint_with_defaulted_set(self):
+        class ClassWithDefaultedSetField(base.VersionedObject):
+            VERSION = 1.0
+            fields = {
+                'empty_default': fields.SetOfIntegersField(default=set()),
+                'non_empty_default': fields.SetOfIntegersField(default={1, 2})
+            }
+        self._add_class(self.obj_classes, ClassWithDefaultedSetField)
+
+        # it is expected that this hash is stable across python versions
+        expected = '1.0-bcc44920f2f727eca463c6eb4fe8445b'
+        actual = self.ovc._get_fingerprint(ClassWithDefaultedSetField.__name__)
+        self.assertEqual(expected, actual)
+
     def test_get_dependencies(self):
         # Make sure _get_dependencies() generates a correct tree when parsing
         # an object
@@ -723,3 +737,29 @@ class TestStableObjectJsonFixture(test.TestCase):
             self.assertEqual(
                 ['a', 'z'],
                 obj.obj_to_primitive()['versioned_object.changes'])
+
+
+class TestMethodSpec(test.TestCase):
+    def setUp(self):
+        super(TestMethodSpec, self).setUp()
+
+        def test_method1(a, b, kw1=123, **kwargs):
+            pass
+
+        def test_method2(a, b, *args):
+            pass
+
+        def test_method3(a, b, *args, kw1=123, **kwargs):
+            pass
+
+        self._test_method1 = test_method1
+        self._test_method2 = test_method2
+        self._test_method3 = test_method3
+
+    def test_method_spec_compat(self):
+        self.assertEqual(inspect.getargspec(self._test_method1),
+                         fixture.get_method_spec(self._test_method1))
+        self.assertEqual(inspect.getargspec(self._test_method2),
+                         fixture.get_method_spec(self._test_method2))
+        self.assertEqual(inspect.getfullargspec(self._test_method3),
+                         fixture.get_method_spec(self._test_method3))
